import plotly.express as px
import numpy as np

class PlotlyAnimation(object):
    def __init__(self, state, steps):
        self.state = state
        self.steps = steps
        self.frames = np.expand_dims(self.state.board, axis=0)
        self.frame_gen()
        self.animation = self.animation_gen()

    def frame_gen(self):
        for _ in range(self.steps):
            self.frames = np.concatenate((self.frames, np.expand_dims(self.state.step(), axis=0)), axis=0)
        return self.frames

    def animation_gen(self):
        fig = px.imshow(self.frames, animation_frame=0, color_continuous_scale=[[0, "white"], [1, "black"]])
        fig.layout.updatemenus[0].buttons[0].args[1]['frame']['duration'] = 30
        fig.layout.updatemenus[0].buttons[0].args[1]['transition']['duration'] = 5
        fig.layout.updatemenus[0].showactive = True
        fig.layout.sliders[0].visible = False
        fig.layout.coloraxis.showscale = False
        fig.layout.xaxis.showticklabels = False
        fig.layout.yaxis.showticklabels = False
        return fig

    def show(self):
        self.animation.show()
